#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <string.h>

#define FORWARD 0
#define STOP 1
#define RIGHT 2
#define LEFT 3

#define THRESHOLD 511
#define SPEED 105
#define ERR 10

void init_motor_driver() {
	DDRC |= (1 << PC0); // IN1
	DDRC |= (1 << PC1); // IN2
	DDRC |= (1 << PC2); // IN3
	DDRC |= (1 << PC3); // IN4
	DDRB |= (1 << PB3); // ENA
	DDRB |= (1 << PB4); // ENB

	PORTC &= ~(1 << PC0);
	PORTC &= ~(1 << PC1);
	PORTC &= ~(1 << PC2);
	PORTC &= ~(1 << PC3);
	PORTB |= (1 << PB3);
	PORTB |= (1 << PB4);
}

void move(int direction) {
	
	switch (direction) {
		case FORWARD:
			PORTC |= (1 << PC1);
			PORTC &= ~(1 << PC0);

			PORTC &= ~(1 << PC3);
			PORTC |= (1 << PC2);
			OCR0A = SPEED;
			OCR0B = SPEED;

			break;
		
		case STOP:
			PORTC &= ~(1 << PC0);
			PORTC &= ~(1 << PC1);

			PORTC &= ~(1 << PC2);
			PORTC &= ~(1 << PC3);

			break;

		case RIGHT:

			OCR0A = SPEED - ERR;
			OCR0B = 0;
			break;
			
		case LEFT:

			OCR0A = 0;
			OCR0B = SPEED - ERR;
			break;
	}
	// _delay_ms(1);
}

void init_timer() {

	OCR0A = 0;
	OCR0B = 0;

    TCCR0A |= (1 << WGM01) | (1 << WGM00); // fast pwm cu top la 0xff
	TCCR0A |= (1 << COM0A1); // non-inverting mode
	TCCR0A |= (1 << COM0B1);
    TCCR0B |= (1 << CS00) | (1 << CS02); // prescaler 1024
}

int check_direction(int values[]) {


	if (values[3] >= THRESHOLD && values[4] >= THRESHOLD) {
		if (values[0] >= THRESHOLD
				|| values[1] >= THRESHOLD
				|| values[2] >= THRESHOLD){
			move(LEFT);
		} else if (values[7] >= THRESHOLD
				|| values[6] >= THRESHOLD
				|| values[5] >= THRESHOLD){
			move(RIGHT);
		} else {
			move(FORWARD);
		}
		return 1;
	} else if (values[0] >= THRESHOLD
				|| values[1] >= THRESHOLD
				|| values[2] >= THRESHOLD){
		move(LEFT);
		return 1;
	} else if (values[7] >= THRESHOLD
				|| values[6] >= THRESHOLD
				|| values[5] >= THRESHOLD) {
		move(RIGHT);
		return 1;
	}

	return 0;
}

int main(void) {
	init_motor_driver();
	init_timer();
	move(FORWARD);

	int channel, values[8], old_values[8];
	memset(old_values, 0, sizeof(old_values));

	while(1) {
		ADMUX = 0;
		ADMUX |= (1 << REFS0);

		channel = 0;
		while (channel <= 7) {
			ADMUX &= 0b11100000;
			ADMUX |= channel;

			ADCSRA = 0;
			ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
			ADCSRA |= (1 << ADEN);
			ADCSRA |= (1 << ADSC);

			while (!(ADCSRA & (1 << ADIF)));

			values[channel] = ADC;

			++channel;
		}

		if(check_direction(values) == 0) {
			move(FORWARD);
		}

	}

	return 0;
}